/**
 * Updates the history behavior of chat boxes.
 */
converse.plugins.add('sib-history-improved', {
  dependencies: [
    'converse-mam',
  ],
  overrides: {
    ChatRoomView: {
      events: {
        'wheel .chat-content__messages': 'onScrolledUp',
      },
    },
    ChatBoxView: {
      events: {
        'wheel .chat-content__messages': 'onScrolledUp',
      },

      /**
       * Fetch the old messages if there is no scroll available.
       * @param {WheelEvent} ev
       */
      onScrolledUp(ev) {
        const { api } = this.__super__._converse;
        const { utils } = converse.env;
        const scrollable = this.msgs_container.clientHeight < this.msgs_container.scrollHeight;

        if (ev.deltaY < 0 && !scrollable) {
          api.trigger('chatBoxScrolledUp', this);
          utils.safeSave(this.model, {
            scrolled: true,
            scrollTop: 0,
          });
        }
      },
    },
    ChatBox: {
      /**
       * Disable this event handler.
       * Causes the history to be fetched twice for private chats.
       */
      afterMessagesFetched() {
      },
    },
  },
  initialize() {
    const _converse = this._converse;
    const { api } = _converse;
    const { log } = converse.env;
    let counter = 0;

    // Forces the loading of the real number of messages (or more)
    // from the setting `archived_messages_page_size`.
    api.listen.on('MAMResult', async data => {

      const max = +api.settings.get('archived_messages_page_size');
      const messages = await Promise.all(data.messages);

      // Increase counter with the messages that contains a body
      counter += messages.filter(attrs => attrs.body).length;

      // Stop if the max value is not specified
      // Stop if there are less messages available than the page size
      // Stop if the counter registers more messages than the max value
      if (!max || messages.length < max || counter >= max) {
        log.debug(`Chatbox has enough messages or cannot get more, count: ${counter}`);
        counter = 0;
        return;
      }

      log.debug(`Chatbox needs more messages, count: ${counter}`);

      const is_groupchat = data.chatbox.get('type') === _converse.CHATROOMS_TYPE;
      const oldest_message = messages[0] || null;

      if (oldest_message) {
        const by_jid = is_groupchat ? data.chatbox.get('jid') : _converse.bare_jid;
        const stanza_id = oldest_message && oldest_message['stanza_id '.concat(by_jid)];

        if (stanza_id) {
          log.debug(`Loading messages before stanza: ${stanza_id}`);
          await data.chatbox.fetchArchivedMessages({
            'before': stanza_id,
          });
        } else {
          log.debug(`Loading messages before time: ${oldest_message['time']}`);
          await data.chatbox.fetchArchivedMessages({
            'end': oldest_message['time'],
          });
        }
      }
    });
  },
});
